package com.redxun.knowledge.album.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.knowledge.album.entity.dao.AlbumCategory;
import com.redxun.knowledge.album.entity.vo.AlbumCategoryVo;
import com.redxun.knowledge.entity.dao.Banner;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 业务--专辑分类表数据库访问层
 */
@Mapper
public interface AlbumCategoryMapper extends BaseDao<AlbumCategory> {

    /**
     * 插入和修改时变更sort
     *
     * @param parent
     * @param sort
     */
    void insertOrUpdateBySort(@Param("parent") String parent, @Param("sort") Integer sort);

    /**
     * 分页查询
     *
     * @param page
     * @param params
     * @param order
     * @return
     */
    IPage findAllAlbumCategory(IPage<AlbumCategory> page, @Param("params") Map<String, Object> params, @Param("order")
            Map<String, Object> order);

    /**
     * 查询一级分类信息
     *
     * @param params
     * @param order
     * @return
     */
    List<AlbumCategoryVo> findAllAlbumCategory1(@Param("params") Map<String, Object> params, @Param("order") Map<String, Object> order);

    /**
     * 查询二级分类信息
     *
     * @param params
     * @param order
     * @return
     */
    List<AlbumCategoryVo> findAllAlbumCategory2(@Param("params") Map<String, Object> params, @Param("order") Map<String, Object> order);

    /**
     * 查询对应级别的专辑分类列表
     *
     * @param level
     * @return
     */
    List<AlbumCategoryVo> listOflevel(Integer level);

    /**
     * 删除专辑分类
     *
     * @param ids
     * @return 专辑Id
     */
    void deleteByPkId(String ids);

    /**
     * 删除专辑分类时变更sort
     *
     * @param parent
     * @param sort
     */
    void deleteBySort(@Param("parent") String parent, @Param("sort") Integer sort);

    /**
     * 根据父Id查询出全部排序信息
     *
     * @param parent
     * @return
     */
    List<Integer> selectSortList(String parent);

    /**
     * 调整分类顺序(同级修改顺序)
     *
     * @param pkId
     * @param sort
     */
    void swapSort(@Param("pkId") String pkId, @Param("sort") int sort);
}
