
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.KnowledgeApprove;
import com.redxun.knowledge.entity.dto.KnowledgeApproveDto;
import com.redxun.knowledge.service.KnowledgeApproveServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.owasp.esapi.util.CollectionsUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/knowledgeApprove")
@Api(tags = "业务--知识审批表")
@ClassDefine(title = "业务--知识审批表", alias = "KnowledgeApproveController", path = "/knowledge/knowledgeApprove", packages = "", packageName = "子系统名称")
public class KnowledgeApproveController extends BaseController<KnowledgeApprove> {

    @Autowired
    KnowledgeApproveServiceImpl knowledgeApproveService;

    @Autowired
    private UserService userService;


    @Override
    public BaseService getBaseService() {
        return knowledgeApproveService;
    }

    @Override
    public String getComment() {
        return "业务--知识审批";
    }

    @ApiOperation(value = "审核管理列表(分页后台)")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllKnowledgeApprove")
    public JsonPageResult findAllKnowledgeApprove(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeApproveService.findAllKnowledgeApprove(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation(value = "知识审核")
    @PostMapping("knowledgeApproveById")
    public JsonResult knowledgeApproveById(@RequestBody KnowledgeApproveDto knowledgeApproveDto) {
        KnowledgeApprove knowledgeApprove = new KnowledgeApprove();
        knowledgeApproveDto.setApprovalTime(new Date());
        knowledgeApproveDto.setActualApprover(userService.queryLoginUsername());
        knowledgeApproveDto.setActualApproverName(userService.queryLoginFullName());
        BeanUtils.copyProperties(knowledgeApproveDto, knowledgeApprove);
        int update = knowledgeApproveService.update(knowledgeApprove);
        if (update == -1) {
            return JsonResult.Success("该知识已被他人审核,审核失败");
        }
        if (update == -2) {
            return JsonResult.Success("审核权限已被修改,审核失败");
        }
        return JsonResult.Success("审核成功");
    }

    @ApiOperation(value = "批量审核")
    @PostMapping("knowledgeApproveBatchById")
    public JsonResult knowledgeApproveBatchById(@RequestBody KnowledgeApproveDto knowledgeApproveDto) {
        List<KnowledgeApprove> knowledgeApproveList = knowledgeApproveDto.getPkIds().stream().map(e -> {
            KnowledgeApprove knowledgeApprove = new KnowledgeApprove();
            knowledgeApprove.setPkId(e);
            knowledgeApprove.setResult(knowledgeApproveDto.getResult());
            knowledgeApprove.setActualApprover(userService.queryLoginUsername());
            knowledgeApprove.setActualApproverName(userService.queryLoginFullName());
            knowledgeApprove.setRemark(knowledgeApproveDto.getRemark());
            knowledgeApprove.setApprovalTime(new Date());
            return knowledgeApprove;
        }).collect(Collectors.toList());
        List<String> result1 = new ArrayList<>();
        List<String> result2 = new ArrayList<>();
        knowledgeApproveList.forEach(e -> {
            int update = knowledgeApproveService.update(e);
            if (update == -1) {
                result1.add(e.getPkId());
            }
            if (update == -2) {
                result2.add(e.getPkId());
            }
        });
        if (!CollectionUtils.isEmpty(result1)) {
            return JsonResult.Success(result1.toString() + "知识已被他人审核,审核失败");
        }
        if (!CollectionUtils.isEmpty(result2)) {
            return JsonResult.Success(result2.toString() + "审核权限已被修改,审核失败");
        }
        return JsonResult.Success("批量审核成功");
    }


}

