package com.redxun.search.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.search.service.SearchService;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.index.qual.GTENegativeOne;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * 文件名: SearchController
 * 作者: Zizg
 * 时间: 2022/12/13
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Slf4j
@RestController
@RequestMapping("/api-knowledge/search")
@Api(tags = "业务--搜索平台")
@ClassDefine(title = "业务--搜索平台", alias = "SearchController", path = "/api-knowledge/search", packages = "", packageName = "子系统名称")
public class SearchController {

    @Autowired
    private SearchService searchService;

    @ApiOperation(value = "业务对文档的全文检索入口")
    @PostMapping("search")
    public Map<String, Object> search(@RequestParam("keyword") String keyword,
                                      @RequestParam(defaultValue = "1") Integer pageIndex,
                                      @RequestParam(defaultValue = "10") Integer pageSize,
                                      @RequestParam("range") Integer range,
                                      @RequestParam("secretType") Integer secretType,
                                      @RequestParam(value = "orderFiled", required = false) Integer orderFiled,
                                      @RequestParam(value = "orderSort", required = false) Integer orderSort,
                                      HttpServletRequest httpServletRequest) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            Map<String, Object> result = searchService.search(keyword, pageIndex, pageSize, range, secretType, orderFiled, orderSort, httpServletRequest);
            return result;
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            log.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return null;
    }

    @ApiOperation("查看文件详情接口")
    @PostMapping("view")
    public Map<String, Object> view(@RequestParam("documentId") String documentId,
                                    @RequestParam("userId") String userId,
                                    @RequestParam("secretType") String secretType,
                                    HttpServletRequest httpServletRequest) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            Map<String, Object> result = searchService.view(documentId, userId, secretType, httpServletRequest);
            return result;
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            log.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return null;
    }

}
